package com.glxp.udi.admin.controller.auth;

import com.glxp.udi.admin.annotation.AuthRuleAnnotation;
import com.glxp.udi.admin.constant.Constant;
import com.glxp.udi.admin.dao.auth.AuthLicenseDao;
import com.glxp.udi.admin.entity.auth.AuthAdmin;
import com.glxp.udi.admin.entity.auth.AuthLicense;
import com.glxp.udi.admin.entity.auth.CustomerInfoEntity;
import com.glxp.udi.admin.entity.info.CompanyEntity;
import com.glxp.udi.admin.exception.JsonException;
import com.glxp.udi.admin.req.auth.LoginRequest;
import com.glxp.udi.admin.req.auth.UpdatePasswordRequest;
import com.glxp.udi.admin.res.auth.LoginUserInfoResponse;
import com.glxp.udi.admin.service.auth.AuthAdminService;
import com.glxp.udi.admin.service.auth.AuthLoginService;
import com.glxp.udi.admin.service.auth.CustomerInfoService;
import com.glxp.udi.admin.service.info.CompanyService;
import com.glxp.udi.admin.util.AppUtils;
import com.glxp.udi.admin.util.IpUtils;
import com.glxp.udi.admin.util.JwtUtils;
import com.glxp.udi.admin.util.PasswordUtils;
import com.glxp.udi.common.enums.ResultEnum;
import com.glxp.udi.common.res.BaseResponse;
import com.glxp.udi.common.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 登录相关
 */
@RestController
@Slf4j
public class LoginController {

    @Autowired
    private AuthLoginService authLoginService;

    @Autowired
    private AuthAdminService authAdminService;

    @Resource
    private CompanyService companyService;

    @Resource
    private AuthLicenseDao authLicenseDao;
    @Resource
    CustomerInfoService customerInfoService;

    /**
     * 用户登录
     *
     * @return
     */
    @PostMapping(value = "/login")
    public BaseResponse index(@RequestBody @Valid LoginRequest loginRequest,
                              BindingResult bindingResult,
                              HttpServletRequest request) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        AuthAdmin authAdmin;
        if (loginRequest.getType() != null && loginRequest.getType() == 1) {
            authAdmin = authAdminService.findByUser(loginRequest.getUsername(), Constant.SYSTEM_CUSTOMER_ID);
        } else {
            authAdmin = authAdminService.findByUserName(loginRequest.getUsername());
            if (authAdmin.getCustomerId() != 110) {
                if (authAdmin.getUserFlag() == 0) {
                    throw new JsonException(ResultEnum.DATA_NOT, "该用户已被禁用！");
                }
                CustomerInfoEntity customerInfoEntity = customerInfoService.selectById(authAdmin.getCustomerId());
                if (customerInfoEntity.getUserFlag() == 0) {
                    throw new JsonException(ResultEnum.DATA_NOT, "该企业账户已被禁用！");
                }
            } else {
                if (authAdmin.getUserFlag() == 0) {
                    throw new JsonException(ResultEnum.DATA_NOT, "该用户已被禁用！");
                }
            }
        }
        if (authAdmin == null) {
            throw new JsonException(ResultEnum.DATA_NOT, "用户名或密码错误");
        }

        if (!PasswordUtils.authAdminPwd(loginRequest.getPassword()).equals(authAdmin.getPassWord())) {
            throw new JsonException(ResultEnum.DATA_NOT, "用户名或密码错误");
        }

        // 更新登录状态
        AuthAdmin authAdminUp = new AuthAdmin();
        authAdminUp.setId(authAdmin.getId());
        authAdminUp.setLastLoginTime(new Date());
        authAdminUp.setLastLoginIp(IpUtils.getIpAddr(request));
        authAdminService.updateAuthAdmin(authAdminUp);

        // 登录成功后获取权限，这里面会设置到缓存
        authLoginService.listRuleByAdminId(authAdmin.getId());

        Map<String, Object> claims = new HashMap<>();
        claims.put("admin_id", authAdmin.getId());
        String token = JwtUtils.createToken(claims, 86400L); // 一天后过期86400L

        Map<String, Object> map = new HashMap<>();
        map.put("id", authAdmin.getId());
        map.put("token", token);

        return ResultVOUtils.success(map);
    }

    /**
     * 获取登录用户信息
     *
     * @return
     */
    @AuthRuleAnnotation("")
    @GetMapping("/admin/auth/login/userInfo")
    public BaseResponse userInfo(HttpServletRequest request) {
        String adminId = request.getHeader("ADMIN_ID");
        Long id = Long.valueOf(adminId);

        AuthAdmin authAdmin = authAdminService.findById(id);

        List<String> authRules = authLoginService.listRuleByAdminId(authAdmin.getId());

        LoginUserInfoResponse loginUserInfoResponse = new LoginUserInfoResponse();
        BeanUtils.copyProperties(authAdmin, loginUserInfoResponse);
        loginUserInfoResponse.setAuthRules(authRules);

        CompanyEntity companyEntity = companyService.findCompany(loginUserInfoResponse.getCustomerId());
        loginUserInfoResponse.setCompanyName(companyEntity.getCompanyName());


        return ResultVOUtils.success(loginUserInfoResponse);
    }

    /**
     * 登出
     *
     * @return
     */
    @PostMapping("/admin/auth/login/out")
    public BaseResponse out() {
        return ResultVOUtils.success();
    }

    /**
     * 修改密码
     *
     * @return
     */
    @AuthRuleAnnotation("") // 需要登录验证,但是不需要权限验证时,value 值填空字符串
    @PostMapping("/admin/auth/login/password")
    public BaseResponse password(@RequestBody @Valid UpdatePasswordRequest updatePasswordRequest,
                                 BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL.getCode(),
                    bindingResult.getFieldError().getDefaultMessage());
        }

        AuthAdmin authAdmin = authAdminService.findById(updatePasswordRequest.getAdminId());
        if (authAdmin == null) {
            throw new JsonException(ResultEnum.DATA_NOT);
        }
        String oldPwd = PasswordUtils.authAdminPwd(updatePasswordRequest.getOldPassword());
        // 旧密码不对
        if (authAdmin.getPassWord() != null
                && !authAdmin.getPassWord().equals(oldPwd)) {
            throw new JsonException(ResultEnum.DATA_NOT, "旧密码匹配失败");
        }

        AuthAdmin authAdminUp = new AuthAdmin();
        authAdminUp.setId(authAdmin.getId());
        String newPwd = PasswordUtils.authAdminPwd(updatePasswordRequest.getNewPassword());
        authAdminUp.setPassWord(newPwd);

        boolean b = authAdminService.updateAuthAdmin(authAdminUp);
        if (b) {
            return ResultVOUtils.success();
        }

        return ResultVOUtils.error(ResultEnum.DATA_CHANGE);
    }

    /**
     * 平台验证
     *
     * @return
     */
    @ResponseBody
    @PostMapping(value = "/verify")
    public BaseResponse verify(@RequestBody Map<String, Object> params,
                               HttpServletRequest request) {
        AuthAdmin authAdmin;
        authAdmin = authAdminService.findByUserName(params.get("username").toString());
        if (authAdmin == null) {
            throw new JsonException(ResultEnum.DATA_NOT, "用户名或密码错误");
        }


        if (!PasswordUtils.authAdminPwd(params.get("password").toString()).equals(authAdmin.getPassWord())) {
            throw new JsonException(ResultEnum.DATA_NOT, "用户名或密码错误");
        }
        AuthLicense authLicense = new AuthLicense();
        if (params.get("appid") != null) {
            authLicense = authLicenseDao.get(params.get("appid").toString());
        } else {
            String appid = AppUtils.getAppid();
            authLicense.setId(appid);
            authLicense.setAppid(appid);
            authLicense.setApiKey(appid);
            if (params.get("name") != null)
                authLicense.setName(params.get("name").toString());
            authLicense.setSecretKey(AppUtils.getSecretKey(appid));
            authLicense.setCustomerId(authAdmin.getCustomerId() + "");
            CompanyEntity companyEntity = companyService.findCompany(authAdmin.getCustomerId());
            authLicense.setCompanyName(companyEntity.getCompanyName());
            authLicense.setCreateDate(new Date());
            authLicenseDao.romveByCustomerId(authAdmin.getCustomerId() + "");
            authLicenseDao.save(authLicense);
        }
        return ResultVOUtils.success(authLicense);
    }

}
